# Generated by Django 4.1.3 on 2022-11-13 05:27

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("jangle", "0001_initial"),
    ]

    operations = [
        migrations.RenameField(
            model_name="regionsubtag",
            old_name="ext_data",
            new_name="region",
        ),
    ]
